# Copyright (c) 2011, Benjamin Lee and contributors
# All rights reserved.
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met:
# * Redistributions of source code must retain the above copyright 
#   notice, this list of conditions and the following disclaimer.
# * Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in the 
#   documentation and/or other materials provided with the distribution.
# 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT 
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR 
# A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT 
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED 
# TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
# $Id$
#
# libfiledb.sh is a shell library for file-backed key-value stores.
#
# libfiledb.sh does not do any locking -- locking must be managed by the
# caller as appropriate.
#
# The on-disk storage format uses the tab character as the field delimiter
# and the new line character as the record delimiter.  Tab characters and
# new lines are accepted in both keys and values and recorded as "\t" and
# "\n" respectively.

# Purpose: Load a filedb into an array
# Inputs: filedb, array
# Prints: N/A
# Returns: 0 if the filedb was loaded into the array, 1 on error
filedb_load()
{
	if [ -z "${1-}" -o -z "${2-}" ]; then
		return 1
	fi

	local filedb="$1"
	local array="$2"

	local key
	local value

	[ -r "${filedb}" ] &&
	local IFS="	"
	while read -r key value; do
		eval "`echo "${array}"`[\"`echo -e "${key}"`\"]=\"`echo -e "${value}"`\""
	done <"${filedb}" &&
	return 0

	return 1
}

# Purpose: Write an array into a filedb
# Inputs: array, filedb
# Prints: N/A
# Returns: 0 if the array was written into the filedb, 1 on error
filedb_write()
{
	if [ -z "${1-}" -o -z "${2-}" ]; then
		return 1
	fi

	local array="$1"
	local filedb="$2"

	local i
	local key
	local value

	[ -w "${filedb}" -o -w "`dirname "${filedb}"`" ] &&
	eval 'for i in "'"\${!${array}"'[@]}"; do
		i="${i//\\/\\\\}"
		i="${i//
/\\n}"
		echo -E "${i}"
	done' | while read -r key; do
		eval "value=\"\${${array}[\"`echo -e "${key}"`\"]}\""
		value="${value//\\/\\\\}"
		value="${value//
/\\n}"
		echo -nE "${key%%	*}"
		while [ "${key}" != "${key#*	}" ]; do
			key="${key#*	}"
			echo -nE "\\t${key%%	*}"
		done
		echo -n "	"
		echo -nE "${value%%	*}"
		while [ "${value}" != "${value#*	}" ]; do
			value="${value#*	}"
			echo -nE "\\t${value%%	*}"
		done
		echo
	done > "${filedb}" &&
	return 0

	return 1
}
